<?php

namespace FiloBlu\Paymentscapture\Helper;

class Data extends \Magento\Framework\App\Helper\AbstractHelper {

    const ADYEN_HPP_METHODS_LIST_PATH = 'filoblu_paymentscapture/hpp_methods_enumeration/hpp_methods';
    const ADYEN_HPP_METHOD_CODE = 'adyen_hpp';

    /**
     * @var \Magento\Sales\Model\ResourceModel\Order\Payment\Transaction\CollectionFactory
     */
    protected $salesResourceModelOrderPaymentTransactionCollectionFactory;

    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Sales\Model\ResourceModel\Order\Payment\Transaction\CollectionFactory $salesResourceModelOrderPaymentTransactionCollectionFactory
    ) {
        $this->salesResourceModelOrderPaymentTransactionCollectionFactory = $salesResourceModelOrderPaymentTransactionCollectionFactory;
        parent::__construct(
            $context
        );
    }

        
    /**
     * 
     * @param \Magento\Sales\Model\Order $order
     * @return boolean
     */
    public function isOrderVoided(\Magento\Sales\Model\Order $order){
        
        $transactions = $this->salesResourceModelOrderPaymentTransactionCollectionFactory->create();
        $transactions->addOrderIdFilter($order->getId());
        $transactions->addTxnTypeFilter('void');
        $transactions->addOrderInformation(array('increment_id'));
        $transactions->addPaymentInformation(array('method'));
        $transactions->load();
        return (bool)$transactions->count();
    }


    /**
     * @param null $websiteId
     * @return array
     */
    public function getHppMethodList($websiteId = null) {
        $preparedHppList = [];
        $hppMethods = $this->getSerializedConfigValue(self::ADYEN_HPP_METHODS_LIST_PATH, $websiteId);
        foreach ($hppMethods as $hppMethod) {
            // NB: I am adding a code prefix: hpp_
            array_push($preparedHppList, array("value" => 'hpp_' . $hppMethod['payment_code'], "label" => "HPP " . $hppMethod['payment_name']));
        }
        uasort($preparedHppList, array($this, 'hppMethodsSortCompare'));
        return $preparedHppList;
    }

    /**
     * Get serialized config value
     * temporarily solution to get unserialized config value
     * should be deprecated in 2.3.x
     * taken from https://nwdthemes.com/2018/06/21/magento-2-working-with-arrayserialized-backend-model/
     *
     * @param $configPath
     * @param null $websiteId
     * @return mixed
     */
    public function getSerializedConfigValue($configPath, $websiteId = null)
    {
        $value = $this->scopeConfig->getValue(
            $configPath,
            \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE,
            $websiteId
        );

        if (empty($value)) return false;

        if (class_exists(\Magento\Framework\Serialize\SerializerInterface::class)) {
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $serializer = $objectManager->create(\Magento\Framework\Serialize\SerializerInterface::class);
            return $serializer->unserialize($value);
        }
        return \unserialize($value);
    }


    private function hppMethodsSortCompare($a, $b) {
        return strcmp($a['label'], $b['label']);
    }


    /**
     * @param $order_payment_method
     * @param $schedule_payment_methods
     * @return bool
     */
    public function paymentMethodMatches($order_payment_method, $schedule_payment_methods) {
        if (count($schedule_payment_methods) == 1 && $schedule_payment_methods[0] == '*') {
            return TRUE;
        }
        if (in_array($order_payment_method['code'], $schedule_payment_methods)) {
            return TRUE;
        }
        if ($order_payment_method['code'] == self::ADYEN_HPP_METHOD_CODE) {
            foreach ($schedule_payment_methods as $schedule_payment_method) {
                if (substr($schedule_payment_method, 0 , 4) == 'hpp_') {
                    $schedule_payment_method_real_name = substr($schedule_payment_method, 4);
                    if ($order_payment_method['cctype'] == $schedule_payment_method_real_name) {
                        return TRUE;
                    }
                }
            }
        }
        return FALSE;
    }
    
}
