<?php

namespace FiloBlu\Paymentscapture\Model;

class Capture extends \Magento\Framework\Model\AbstractModel {

    const VOIDED_ORDER_CHECK_CONFIG_PATH = 'filoblu_paymentscapture/general/check_void';
    const CAPTURE_ACTIVE_CONFIG_PATH = 'filoblu_paymentscapture/general/active';
    const CAPTURE_DEBUG_CONFIG_PATH = 'filoblu_paymentscapture/general/debug';
    const DEBUG_LOGFILE = 'paymentscapture_capture.log';
    const ADYEN_HPP_PAYMENT_CODE = 'adyen_hpp';
    const ORDER_ADYENHPP_PAYMENT_METHODS_CONFIG_PATH = 'filoblu_paymentscapture/general/adyenhpp_payment_method';
    const SCHEDULED_OPERATIONS_CONFIG_PATH = 'filoblu_paymentscapture/general/scheduled_operations';

    protected $_helper;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var \Magento\Sales\Model\ResourceModel\Order\CollectionFactory
     */
    protected $salesResourceModelOrderCollectionFactory;

    /**
     * @var \Magento\Framework\DB\TransactionFactory
     */
    protected $transactionFactory;

    /**
     * @var \FiloBlu\Paymentscapture\Helper\Data
     */
    protected $paymentscaptureHelper;

    /**
     * @var \Magento\Sales\Model\Service\InvoiceService
     */
    protected $invoiceService;

    public function __construct(
    \Magento\Framework\Model\Context $context, \Magento\Framework\Registry $registry, \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig, \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $salesResourceModelOrderCollectionFactory, \Magento\Framework\DB\TransactionFactory $transactionFactory, \FiloBlu\Paymentscapture\Helper\Data $paymentscaptureHelper, \Magento\Sales\Model\Service\InvoiceService $invoiceService, \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null, \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null, array $data = array()) {
        $this->scopeConfig = $scopeConfig;
        $this->salesResourceModelOrderCollectionFactory = $salesResourceModelOrderCollectionFactory;
        $this->transactionFactory = $transactionFactory;
        $this->paymentscaptureHelper = $paymentscaptureHelper;
        $this->invoiceService = $invoiceService;
        parent::__construct(
                $context, $registry, $resource, $resourceCollection, $data
        );
    }


    /**
     * @param $website
     * @param $scheduled_operation
     * @return bool|\Magento\Sales\Model\ResourceModel\Order\Collection
     */
    protected function _getOrdersCollection($website, $scheduled_operation) {
        $initial_statuses = $scheduled_operation['initial_status'];
        if (count($initial_statuses) > 0) {
            $collection = $this->salesResourceModelOrderCollectionFactory->create()->addAttributeToFilter('store_id', array('in' => $website->getStoreIds()));

            if ($scheduled_operation['delay']) {
                $order_age_in_minutes = $scheduled_operation['delay'];
                $date = new \DateTime('@' . mktime(date("H"), date("i") - $order_age_in_minutes, 0, date("m"), date("d"), date("Y")));
                $collection->addAttributeToFilter('created_at', array('lteq' => $date->format('Y-m-d H:i:s')));
            }

            if (!in_array("*", $initial_statuses)) {
                $collection->addAttributeToFilter('status', array('in' => $initial_statuses));
            }

            return $collection;
        } else {
            return false;
        }
    }
    
    
    protected function _isAdyenHppMethodEnabled($order) {
        $isEnabled = FALSE;
        $enabled_adyenhpp_payment_methods = explode(",", $this->scopeConfig->getValue(self::ORDER_ADYENHPP_PAYMENT_METHODS_CONFIG_PATH, \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $order->getStoreId()));
        if (count($enabled_adyenhpp_payment_methods) == 1 && $enabled_adyenhpp_payment_methods[0] == '') {
            $enabled_adyenhpp_payment_methods = array();
        }
        $additionalInformation = $order->getPayment()->getAdditionalInformation();
        if (!is_null($additionalInformation) && isset($additionalInformation['brand_code'])) {
            if (in_array($additionalInformation['brand_code'], $enabled_adyenhpp_payment_methods)) {
                $isEnabled = TRUE;
            }
        }

        return $isEnabled;
    }

    /**
     * 
     * @param \Magento\Sales\Model\Order $order
     * @return boolean
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function _captureOnlinePayment($order) {

        try {

            //added in order to do not capture orders just voided
            if ($this->scopeConfig->getValue(self::VOIDED_ORDER_CHECK_CONFIG_PATH, \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $order->getStoreId()) && $this->paymentscaptureHelper->isOrderVoided($order)) {
                throw new \Magento\Framework\Exception\LocalizedException(__('order transaction just voided or canceled'));
            }


            if (!$order->canInvoice()) {
                throw new \Magento\Framework\Exception\LocalizedException(__('cannot be invoiced'));
            }

            $invoice = $this->invoiceService->prepareInvoice($order);

            if (!$invoice->getTotalQty()) {
                throw new \Magento\Framework\Exception\LocalizedException(__('cannot create an invoice without products.'));
            }

            $invoice->setRequestedCaptureCase(\Magento\Sales\Model\Order\Invoice::CAPTURE_ONLINE);

            $invoice->register();
            $transactionSave = $this->transactionFactory->create()
                    ->addObject($invoice)
                    ->addObject($invoice->getOrder());
            $transactionSave->save();
        } catch (\Exception $e) {
            $this->debugLog($order->getIncrementId() . ' ' . $e->getMessage());
            //$this->_sendFailedEmail($order->getIncrementId(), $e->getMessage());
            return false;
        }

        return true;
    }

    /**
     * 
     * @param type $IncrementId
     * @param type $errorMsg
     * @return type
     * @todo make this work in M2
     */
    protected function _sendFailedEmail($IncrementId, $errorMsg) {

        if (!$this->scopeConfig->getValue('filoblu_paymentscapture/general/error_email_active', \Magento\Store\Model\ScopeInterface::SCOPE_STORE))
            return;

        $mail = Mage::getModel('core/email');
        //$mail->setToName('Your Name');
        $mail->setToEmail($this->scopeConfig->getValue('filoblu_paymentscapture/general/error_email', \Magento\Store\Model\ScopeInterface::SCOPE_STORE));
        $mail->setBody('Capture Error #' . $IncrementId . ' | Reason: ' . $errorMsg);
        $mail->setSubject('Capture Error ' . $IncrementId);
        $mail->setFromEmail($this->scopeConfig->getValue('trans_email/ident_custom1/email', \Magento\Store\Model\ScopeInterface::SCOPE_STORE));
        $mail->setFromName("Capture Error Notification");
        $mail->setType('text');
        try {
            $mail->send();
        } catch (\Exception $e) {
            ;
        }
    }

    public function capture() {

        /** @var \Magento\Framework\App\ObjectManager $om */
        $om = \Magento\Framework\App\ObjectManager::getInstance();
        /** @var \Magento\Store\Model\StoreManagerInterface|\Magento\Store\Model\StoreManager $storeManager */
        $storeManager = $om->get('Magento\Store\Model\StoreManagerInterface');
        $websites = $storeManager->getWebsites();
        foreach ($websites as $website) {
            if (!$this->scopeConfig->isSetFlag(self::CAPTURE_ACTIVE_CONFIG_PATH, \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE, $website)) {
                continue;
            }
            $scheduledOperations = $this->paymentscaptureHelper->getSerializedConfigValue(self::SCHEDULED_OPERATIONS_CONFIG_PATH, $website);
            if (!is_array($scheduledOperations) || empty($scheduledOperations)) {
                continue;
            }

            foreach ($scheduledOperations as $scheduledOperation) {
                $orders = $this->_getOrdersCollection($website, $scheduledOperation);

                foreach ($orders as $order) {
                    $this->debugLog('Processing order #' . $order->getIncrementId());

                    //check payment method
                    $orderPaymentMethod = array('code' => $order->getPayment()->getMethod(), 'cctype' => $order->getPayment()->getCcType());
                    if (!$this->paymentscaptureHelper->paymentMethodMatches($orderPaymentMethod, $scheduledOperation['payment_methods'])) {
                        continue;
                    }
                    //check if capture is needed ? capture
                    $isCaptureRequired = $scheduledOperation["is_capture_required"];
                    if ($isCaptureRequired) {
                        $isCaptured = $this->_captureOnlinePayment($order);
                    } else {
                        $isCaptured = TRUE;
                    }
                    //check if order status (after capture) is ok with final status
                    if ($isCaptured) {
                        $finalStatus = ($scheduledOperation['final_status'] && $scheduledOperation['final_status'] != '*') ? $scheduledOperation['final_status'] : FALSE;
                        if ($finalStatus != FALSE && $order->getStatus() != $finalStatus) {
                            $order->setState('processing');
                            $order->addStatusHistoryComment("Order processed by Payment capture cronJob", $finalStatus);
                            $order->save();
                        } else {
                            $order->addStatusHistoryComment("Order processed by Payment capture cronJob (no status modification needed)");
                            $order->save();
                        }
                    } else {
                        $this->debugLog('Order #' . $order->getIncrementId() . ' not captured');
                    }
                }
            }
        }
    }

    
    protected function debugLog($message) {
        if ($this->scopeConfig->isSetFlag(self::CAPTURE_DEBUG_CONFIG_PATH)) {
            $writer = new \Zend\Log\Writer\Stream(BP . '/var/log/' . self::DEBUG_LOGFILE);
            $logger = new \Zend\Log\Logger();
            $logger->addWriter($writer);
            $logger->info($message);
        }
    }

}
